@@ -19,46 +19,63 @@ import logging
 
 logger = logging.getLogger(__name__)
 
+"""
+Attributes:
+--------------
+microgrid: Union[Microgrid.Microgrid, modular_microgrid.ModularMicrogrid]
+    The underlying microgrid
+
+horizon: int
+    The forecast horizon being used in MPC
+
+has_genset: bool
+    Whether the microgrid has a genset or not
+
+p_vars: cvxpy.Variable, shape ((7+self.has_genset)*horizon,)
+    Vector of all of the controls, at all timesteps. See P in pymgrid paper for details.
+
+u_genset: None or cvxpy.Variable, shape (self.horizon,)
+    Boolean vector variable denoting the status of the genset (on or off) at each timestep if
+    the genset exists. If not genset, u_genset = None.
+
+costs: cvxpy.Parameter, shape ((7+self.has_genset)*self.horizon,)
+    Parameter vector of all of the respective costs, at all timesteps. See C in pymgrid paper for details.
+
+equality_rhs: cvxpy.Parameter, shape (2 * self.horizon,)
+    Parameter vector contraining the RHS of the equality constraint equation. See b in pymgrid paper for details.
+
+inequality_rhs: cvxpy.Parameter, shape (8 * self.horizon,)
+    Parameter vector contraining the RHS of the inequality constraint equation. See d in pymgrid paper for details.
+
+problem: cvxpy.problems.problem.Problem
+    The constraint optimization problem to solve
+"""
+
 
 class ModelPredictiveControl:
-
     """
-    A class to run Model Predictive Control using the model outlined in the pymgrid paper
+    Run a model predictive control algorithm on a microgrid.
 
-    Parameters:
-        microgrid: Microgrid.Microgrid
-            The underlying microgrid on which MPC will be run
+    In model predictive control, a model of the microgrid is used to predict the microgrid's response to taking
+    certain actions. Armed with this prediction model, we can predict the microgrid's response to simulating forward
+    a certain number of steps (the forecast "horizon"). This results in an objective function -- with the objective
+    being the cost of running the microgrid over the entire horizon.
 
-    Attributes:
-    --------------
-    microgrid: Union[Microgrid.Microgrid, modular_microgrid.ModularMicrogrid]
-        The underlying microgrid
+    Given the solution of this optimization problem, we apply the control we found at the current step (ignoring the
+    rest) and then repeat.
 
-    horizon: int
-        The forecast horizon being used in MPC
+    The specifics of the model implementation can be seen in the accompanying paper.
 
-    has_genset: bool
-        Whether the microgrid has a genset or not
+    .. warning::
+       This implementation of model predictive control does not support arbitrary microgrid components. One each
+       of load, renewable, battery, grid, and genset are allowed. Microgrids are not required to have both grid and
+       genset but they must have one; they also must have one each of load, renewable, and battery.
 
-    p_vars: cvxpy.Variable, shape ((7+self.has_genset)*horizon,)
-        Vector of all of the controls, at all timesteps. See P in pymgrid paper for details.
-
-    u_genset: None or cvxpy.Variable, shape (self.horizon,)
-        Boolean vector variable denoting the status of the genset (on or off) at each timestep if
-        the genset exists. If not genset, u_genset = None.
-
-    costs: cvxpy.Parameter, shape ((7+self.has_genset)*self.horizon,)
-        Parameter vector of all of the respective costs, at all timesteps. See C in pymgrid paper for details.
-
-    equality_rhs: cvxpy.Parameter, shape (2 * self.horizon,)
-        Parameter vector contraining the RHS of the equality constraint equation. See b in pymgrid paper for details.
-
-    inequality_rhs: cvxpy.Parameter, shape (8 * self.horizon,)
-        Parameter vector contraining the RHS of the inequality constraint equation. See d in pymgrid paper for details.
-
-    problem: cvxpy.problems.problem.Problem
-        The constraint optimization problem to solve
+    Parameters
+    ----------
 
+    microgrid : :class:`pymgrid.Microgrid`
+        Microgrid on which to run model predictive control.
 
     """
     def __init__(self, microgrid, solver=None):
@@ -87,6 +104,9 @@ class ModelPredictiveControl:
 
     @property
     def has_genset(self):
+        """
+        :meta private:
+        """
         if self.is_modular:
             return "genset" in self.microgrid_module_names.keys()
         else:
@@ -112,7 +132,8 @@ class ModelPredictiveControl:
             if suf and s.endswith(suf):
                 return s[:-len(suf)]
             return s
-        return {remove_suffix(module.item().__class__.__name__, "Module").lower(): name for name, module in modular_microgrid.iterdict()}
+        return {remove_suffix(module.item().__class__.__name__, "Module").lower(): name
+                for name, module in modular_microgrid.modules.iterdict()}
 
     def _get_horizon(self):
         if self.is_modular:
@@ -182,7 +203,7 @@ class ModelPredictiveControl:
         battery_capacity = battery.max_capacity
         cost_battery_cycle = battery.battery_cost_cycle
 
-        cost_loss_load = self.microgrid.modules[self.microgrid_module_names["load"]].item().loss_load_cost
+        cost_loss_load = self.microgrid.modules[self.microgrid_module_names["unbalancedenergy"]].item().loss_load_cost
 
         if self.has_genset:
             genset = self.microgrid.modules[self.microgrid_module_names["genset"]].item()
@@ -207,7 +228,6 @@ class ModelPredictiveControl:
             genset_co2
         )
 
-
     def _create_problem(self, eta, battery_capacity, fuel_cost, cost_battery_cycle, cost_loss_load,
                         p_genset_min, p_genset_max, cost_co2, genset_co2):
 
@@ -383,7 +403,7 @@ class ModelPredictiveControl:
                         p_max_import, p_max_export, soc_0, p_genset_max, cost_co2, grid_co2, genset_co2,):
 
         """
-        Protected, called by set_and_solve.
+        Protected, called by _set_and_solve.
         Sets the time-varying (and some static) parameters in the optimization problem at any given timestep.
 
         :param load_vector: np.ndarray, shape (self.horizon,)
@@ -471,7 +491,218 @@ class ModelPredictiveControl:
         if np.isnan(self.costs.value).any():
             raise RuntimeError('There are still nan values in self.costs.value, something is wrong')
 
-    def set_and_solve(self,
+    def run(self, max_steps=None, verbose=False):
+        """
+        Run the model prediction control algorithm.
+
+        Parameters
+        ---------
+        max_steps : int or None, default None
+            Maximum number of MPC steps. If None, run until the microgrid terminates.
+
+        verbose : bool, default False
+            Whether to display a progress bar.
+
+        Returns
+        -------
+        log : pd.DataFrame
+            Results of running the rule-based control algorithm.
+
+        """
+        if self.is_modular:
+            return self._run_mpc_on_modular(forecast_steps=max_steps, verbose=verbose)
+        else:
+            return self._run_mpc_on_nonmodular(forecast_steps=max_steps, verbose=verbose)
+
+    def _run_mpc_on_nonmodular(self, forecast_steps=None, verbose=False):
+        """
+        Function that allows MPC to be run on self.microgrid by first parsing its data
+
+        :param forecast_steps: int, default None
+            Number of steps to run MPC on. If None, runs over 8760-self.horizon steps
+        :param verbose: bool
+            Whether to discuss progress
+        :return:
+            output, ControlOutput
+                dict-like containing the DataFrames ('action', 'status', 'production', 'cost'),
+                but with an ordering defined via comparing the costs.
+        """
+        sample = return_underlying_data(self.microgrid)
+        sample = sample.reset_index(drop=True)
+        return self._run_mpc_on_sample(sample, forecast_steps=forecast_steps, verbose=verbose)
+
+    def _run_mpc_on_modular(self, forecast_steps=None, verbose=False):
+
+        if forecast_steps is None:
+            num_iter = len(self.microgrid) - self.horizon
+        else:
+            assert forecast_steps <= len(self.microgrid) - self.horizon, 'forecast steps can\'t look past horizon'
+            num_iter = forecast_steps
+
+        self.microgrid.reset()
+
+        for i in tqdm(range(num_iter), desc="MPC Progress", disable=(not verbose)):
+            control = self._set_and_solve(*self._get_modular_state_values(),
+                                         iteration=i,
+                                         total_iterations=num_iter)
+
+            self.microgrid.run(control, normalized=False)
+
+        return self.microgrid.get_log()
+
+    def _run_mpc_on_sample(self, sample, forecast_steps=None, verbose=False):
+        """
+        Runs MPC on a sample over a number of iterations
+
+        :param sample: pd.DataFrame, shape (8760,3)
+            sample to run the MPC on. Must contain columns 'load', 'pv', and 'grid'.
+        :param forecast_steps: int, default None
+            Number of steps to run MPC on. If None, runs over 8760-self.horizon steps
+        :param verbose: bool
+            Whether to discuss progress
+        :return:
+            output, ControlOutput
+                dict-like containing the DataFrames ('action', 'status', 'production', 'cost'),
+                but with an ordering defined via comparing the costs.
+        """
+        if not isinstance(sample, pd.DataFrame):
+            raise TypeError('sample must be of type pd.DataFrame, is {}'.format(type(sample)))
+
+        sample = sample.iloc[:self.microgrid._data_length]
+
+        # dataframes, copied API from _baseline_linprog
+        self.microgrid.reset()
+        baseline_linprog_action = deepcopy(self.microgrid._df_record_control_dict)
+        baseline_linprog_update_status = deepcopy(self.microgrid._df_record_state)
+        baseline_linprog_record_production = deepcopy(self.microgrid._df_record_actual_production)
+        baseline_linprog_cost = deepcopy(self.microgrid._df_record_cost)
+        baseline_linprog_co2 = deepcopy(self.microgrid._df_record_co2)
+
+        T = len(sample)
+        horizon = self.microgrid.horizon
+
+        if forecast_steps is None:
+            num_iter = T - horizon
+        else:
+            assert forecast_steps <= T - horizon, 'forecast steps can\'t look past horizon'
+            num_iter = forecast_steps
+
+        t0 = time.time()
+        old_control_dict = None
+
+        for i in tqdm(range(num_iter), desc="MPC Progress", disable=(not verbose)):
+
+            if self.microgrid.architecture['grid'] == 0:
+                temp_grid = np.zeros(horizon)
+                price_import = np.zeros(horizon)
+                price_export = np.zeros(horizon)
+                p_max_import = 0
+                p_max_export = 0
+                grid_co2 = np.zeros(horizon)
+            else:
+                temp_grid = sample.loc[i:i + horizon - 1, 'grid'].values
+                price_import = self.microgrid._grid_price_import.iloc[i:i + horizon].values
+                price_export = self.microgrid._grid_price_export.iloc[i:i + horizon].values
+                grid_co2 = self.microgrid._grid_co2.iloc[i:i + horizon].values
+                p_max_import = self.microgrid.parameters['grid_power_import'].values[0]
+                p_max_export = self.microgrid.parameters['grid_power_export'].values[0]
+
+                if temp_grid.shape != price_export.shape and price_export.shape != price_import.shape:
+                    raise RuntimeError('I think this is a problem')
+
+
+            e_min = self.microgrid.parameters['battery_soc_min'].values[0]
+            e_max = self.microgrid.parameters['battery_soc_max'].values[0]
+            p_max_charge = self.microgrid.parameters['battery_power_charge'].values[0]
+            p_max_discharge = self.microgrid.parameters['battery_power_discharge'].values[0]
+
+            soc_0 = baseline_linprog_update_status['battery_soc'][-1]
+
+            cost_co2 = self.microgrid.parameters['cost_co2'].values[0]
+
+            if self.has_genset:
+                p_genset_max = self.microgrid.parameters['genset_pmax'].values[0] *\
+                           self.microgrid.parameters['genset_rated_power'].values[0]
+                genset_co2 = self.microgrid.parameters['genset_co2'].values[0]
+            else:
+                p_genset_max = None
+                genset_co2 = None
+
+            # Solve one step of MPC
+            control_dict = self._set_and_solve(sample.loc[i:i + horizon - 1, 'load'].values,
+                                              sample.loc[i:i + horizon - 1, 'pv'].values, temp_grid, price_import,
+                                              price_export, e_max, e_min, p_max_charge, p_max_discharge, p_max_import,
+                                              p_max_export, soc_0, p_genset_max, cost_co2, grid_co2, genset_co2, iteration = i, total_iterations = num_iter)
+
+            if control_dict is not None:
+                baseline_linprog_action = self.microgrid._record_action(control_dict, baseline_linprog_action)
+                baseline_linprog_record_production = self.microgrid._record_production(control_dict,
+                                                                                       baseline_linprog_record_production,
+                                                                                       baseline_linprog_update_status)
+                old_control_dict = control_dict.copy()
+
+            elif old_control_dict is not None:
+                print('Using previous controls')
+                baseline_linprog_action = self.microgrid._record_action(old_control_dict, baseline_linprog_action)
+                baseline_linprog_record_production = self.microgrid._record_production(old_control_dict,
+                                                                                       baseline_linprog_record_production,
+                                                                                       baseline_linprog_update_status)
+            else:
+                raise RuntimeError('Fell through, was unable to solve for control_dict and could not find previous control dict')
+
+            if self.microgrid.architecture['grid'] == 1:
+                baseline_linprog_co2 = self.microgrid._record_co2(
+                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
+                    baseline_linprog_co2,
+                    self.microgrid._grid_co2.iloc[i].values[0],
+                )
+
+                baseline_linprog_update_status = self.microgrid._update_status(
+                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
+                    baseline_linprog_update_status,
+                    sample.at[i + 1, 'load'],
+                    sample.at[i + 1, 'pv'],
+                    sample.at[i + 1, 'grid'],
+                    self.microgrid._grid_price_import.iloc[i + 1].values[0],
+                    self.microgrid._grid_price_export.iloc[i + 1].values[0],
+                    self.microgrid._grid_co2.iloc[i + 1].values[0],
+                )
+
+                baseline_linprog_cost = self.microgrid._record_cost(
+                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
+                    baseline_linprog_cost,
+                    baseline_linprog_co2,
+                    self.microgrid._grid_price_import.iloc[i, 0],
+                    self.microgrid._grid_price_export.iloc[i, 0])
+            else:
+                baseline_linprog_co2 = self.microgrid._record_co2(
+                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
+                    baseline_linprog_co2,
+                )
+
+                baseline_linprog_update_status = self.microgrid._update_status(
+                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
+                    baseline_linprog_update_status,
+                    sample.at[i + 1, 'load'],
+                    sample.at[i + 1, 'pv']
+                )
+                baseline_linprog_cost = self.microgrid._record_cost(
+                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
+                    baseline_linprog_cost,
+                    baseline_linprog_co2,
+                )
+
+        names = ('action', 'status', 'production', 'cost', 'co2')
+
+        dfs = (baseline_linprog_action, baseline_linprog_update_status,
+               baseline_linprog_record_production, baseline_linprog_cost, baseline_linprog_co2)
+
+        if verbose:
+            print('Total time: {} minutes'.format(round((time.time()-t0)/60, 2)))
+
+        return ControlOutput(names, dfs, 'mpc')
+
+    def _set_and_solve(self,
                       load_vector,
                       pv_vector,
                       grid_vector,
@@ -636,8 +867,6 @@ class ModelPredictiveControl:
         grid_import, grid_export = control_vals[0:2]
         grid_diff = grid_import - grid_export
 
-        load = load_vector[0]
-
         if battery_charge > 0 and battery_discharge > 0:
             warn(f"battery_charge={battery_charge} and battery_discharge={battery_discharge} are both nonzero. "
                  f"Flattening to the difference, leading to a {'discharge' if battery_diff > 0 else 'charge'} of {battery_diff}.")
@@ -649,213 +878,10 @@ class ModelPredictiveControl:
         if "grid" in self.microgrid_module_names.keys():
             control.update({self.microgrid_module_names["grid"]: grid_diff})
 
-        control.update({self.microgrid_module_names["battery"]: battery_diff,
-                        self.microgrid_module_names["load"]: -1.0 * load})
+        control.update({self.microgrid_module_names["battery"]: battery_diff})
 
         return control
 
-    def run_mpc_on_sample(self, sample, forecast_steps=None, verbose=False):
-        """
-        Runs MPC on a sample over a number of iterations
-
-        :param sample: pd.DataFrame, shape (8760,3)
-            sample to run the MPC on. Must contain columns 'load', 'pv', and 'grid'.
-        :param forecast_steps: int, default None
-            Number of steps to run MPC on. If None, runs over 8760-self.horizon steps
-        :param verbose: bool
-            Whether to discuss progress
-        :return:
-            output, ControlOutput
-                dict-like containing the DataFrames ('action', 'status', 'production', 'cost'),
-                but with an ordering defined via comparing the costs.
-        """
-        if not isinstance(sample, pd.DataFrame):
-            raise TypeError('sample must be of type pd.DataFrame, is {}'.format(type(sample)))
-
-        sample = sample.iloc[:self.microgrid._data_length]
-
-        # dataframes, copied API from _baseline_linprog
-        self.microgrid.reset()
-        baseline_linprog_action = deepcopy(self.microgrid._df_record_control_dict)
-        baseline_linprog_update_status = deepcopy(self.microgrid._df_record_state)
-        baseline_linprog_record_production = deepcopy(self.microgrid._df_record_actual_production)
-        baseline_linprog_cost = deepcopy(self.microgrid._df_record_cost)
-        baseline_linprog_co2 = deepcopy(self.microgrid._df_record_co2)
-
-        T = len(sample)
-        horizon = self.microgrid.horizon
-
-        if forecast_steps is None:
-            num_iter = T - horizon
-        else:
-            assert forecast_steps <= T - horizon, 'forecast steps can\'t look past horizon'
-            num_iter = forecast_steps
-
-        t0 = time.time()
-        old_control_dict = None
-
-        for i in tqdm(range(num_iter), desc="MPC Progress", disable=(not verbose)):
-
-            if self.microgrid.architecture['grid'] == 0:
-                temp_grid = np.zeros(horizon)
-                price_import = np.zeros(horizon)
-                price_export = np.zeros(horizon)
-                p_max_import = 0
-                p_max_export = 0
-                grid_co2 = np.zeros(horizon)
-            else:
-                temp_grid = sample.loc[i:i + horizon - 1, 'grid'].values
-                price_import = self.microgrid._grid_price_import.iloc[i:i + horizon].values
-                price_export = self.microgrid._grid_price_export.iloc[i:i + horizon].values
-                grid_co2 = self.microgrid._grid_co2.iloc[i:i + horizon].values
-                p_max_import = self.microgrid.parameters['grid_power_import'].values[0]
-                p_max_export = self.microgrid.parameters['grid_power_export'].values[0]
-
-                if temp_grid.shape != price_export.shape and price_export.shape != price_import.shape:
-                    raise RuntimeError('I think this is a problem')
-
-
-            e_min = self.microgrid.parameters['battery_soc_min'].values[0]
-            e_max = self.microgrid.parameters['battery_soc_max'].values[0]
-            p_max_charge = self.microgrid.parameters['battery_power_charge'].values[0]
-            p_max_discharge = self.microgrid.parameters['battery_power_discharge'].values[0]
-
-            soc_0 = baseline_linprog_update_status['battery_soc'][-1]
-
-            cost_co2 = self.microgrid.parameters['cost_co2'].values[0]
-
-            if self.has_genset:
-                p_genset_max = self.microgrid.parameters['genset_pmax'].values[0] *\
-                           self.microgrid.parameters['genset_rated_power'].values[0]
-                genset_co2 = self.microgrid.parameters['genset_co2'].values[0]
-            else:
-                p_genset_max = None
-                genset_co2 = None
-
-            # Solve one step of MPC
-            control_dict = self.set_and_solve(sample.loc[i:i + horizon - 1, 'load'].values,
-                                              sample.loc[i:i + horizon - 1, 'pv'].values, temp_grid, price_import,
-                                              price_export, e_max, e_min, p_max_charge, p_max_discharge, p_max_import,
-                                              p_max_export, soc_0, p_genset_max, cost_co2, grid_co2, genset_co2, iteration = i, total_iterations = num_iter)
-
-            if control_dict is not None:
-                baseline_linprog_action = self.microgrid._record_action(control_dict, baseline_linprog_action)
-                baseline_linprog_record_production = self.microgrid._record_production(control_dict,
-                                                                                       baseline_linprog_record_production,
-                                                                                       baseline_linprog_update_status)
-                old_control_dict = control_dict.copy()
-
-            elif old_control_dict is not None:
-                print('Using previous controls')
-                baseline_linprog_action = self.microgrid._record_action(old_control_dict, baseline_linprog_action)
-                baseline_linprog_record_production = self.microgrid._record_production(old_control_dict,
-                                                                                       baseline_linprog_record_production,
-                                                                                       baseline_linprog_update_status)
-            else:
-                raise RuntimeError('Fell through, was unable to solve for control_dict and could not find previous control dict')
-
-            if self.microgrid.architecture['grid'] == 1:
-                baseline_linprog_co2 = self.microgrid._record_co2(
-                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
-                    baseline_linprog_co2,
-                    self.microgrid._grid_co2.iloc[i].values[0],
-                )
-
-                baseline_linprog_update_status = self.microgrid._update_status(
-                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
-                    baseline_linprog_update_status,
-                    sample.at[i + 1, 'load'],
-                    sample.at[i + 1, 'pv'],
-                    sample.at[i + 1, 'grid'],
-                    self.microgrid._grid_price_import.iloc[i + 1].values[0],
-                    self.microgrid._grid_price_export.iloc[i + 1].values[0],
-                    self.microgrid._grid_co2.iloc[i + 1].values[0],
-                )
-
-                baseline_linprog_cost = self.microgrid._record_cost(
-                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
-                    baseline_linprog_cost,
-                    baseline_linprog_co2,
-                    self.microgrid._grid_price_import.iloc[i, 0],
-                    self.microgrid._grid_price_export.iloc[i, 0])
-            else:
-                baseline_linprog_co2 = self.microgrid._record_co2(
-                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
-                    baseline_linprog_co2,
-                )
-
-                baseline_linprog_update_status = self.microgrid._update_status(
-                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
-                    baseline_linprog_update_status,
-                    sample.at[i + 1, 'load'],
-                    sample.at[i + 1, 'pv']
-                )
-                baseline_linprog_cost = self.microgrid._record_cost(
-                    {i: baseline_linprog_record_production[i][-1] for i in baseline_linprog_record_production},
-                    baseline_linprog_cost,
-                    baseline_linprog_co2,
-                )
-
-        names = ('action', 'status', 'production', 'cost', 'co2')
-
-        dfs = (baseline_linprog_action, baseline_linprog_update_status,
-               baseline_linprog_record_production, baseline_linprog_cost, baseline_linprog_co2)
-
-        if verbose:
-            print('Total time: {} minutes'.format(round((time.time()-t0)/60, 2)))
-
-        return ControlOutput(names, dfs, 'mpc')
-
-    def run_mpc_on_microgrid(self, forecast_steps=None, verbose=False, **kwargs):
-        """
-        Function that allows MPC to be run on self.microgrid by first parsing its data
-
-        :param forecast_steps: int, default None
-            Number of steps to run MPC on. If None, runs over 8760-self.horizon steps
-        :param verbose: bool
-            Whether to display progress bar
-        """
-        if self.is_modular:
-            return self.run_mpc_on_modular(forecast_steps=forecast_steps, verbose=verbose)
-        else:
-            return self.run_mpc_on_nonmodular(forecast_steps=forecast_steps, verbose=verbose)
-
-    def run_mpc_on_nonmodular(self, forecast_steps=None, verbose=False):
-        """
-        Function that allows MPC to be run on self.microgrid by first parsing its data
-
-        :param forecast_steps: int, default None
-            Number of steps to run MPC on. If None, runs over 8760-self.horizon steps
-        :param verbose: bool
-            Whether to discuss progress
-        :return:
-            output, ControlOutput
-                dict-like containing the DataFrames ('action', 'status', 'production', 'cost'),
-                but with an ordering defined via comparing the costs.
-        """
-        sample = return_underlying_data(self.microgrid)
-        sample = sample.reset_index(drop=True)
-        return self.run_mpc_on_sample(sample, forecast_steps=forecast_steps, verbose=verbose)
-
-    def run_mpc_on_modular(self, forecast_steps=None, verbose=False):
-
-        if forecast_steps is None:
-            num_iter = len(self.microgrid) - self.horizon
-        else:
-            assert forecast_steps <= len(self.microgrid) - self.horizon, 'forecast steps can\'t look past horizon'
-            num_iter = forecast_steps
-
-        self.microgrid.reset()
-
-        for i in tqdm(range(num_iter), desc="MPC Progress", disable=(not verbose)):
-            control = self.set_and_solve(*self._get_modular_state_values(),
-                                         iteration=i,
-                                         total_iterations=num_iter)
-
-            self.microgrid.run(control, normalized=False)
-
-        return self.microgrid.get_log()
-
     def _get_modular_state_values(self):
 
         load_state = -1.0 * self.microgrid.modules[self.microgrid_module_names["load"]].item().state # state is negative, want positive values.
@@ -874,9 +900,9 @@ class ModelPredictiveControl:
         else:
             grid_status = np.ones(self.horizon)
 
-            price_import = grid.import_price()
-            price_export = grid.export_price()
-            grid_co2_per_kwh = grid.co2_per_kwh()
+            price_import = grid.import_price
+            price_export = grid.export_price
+            grid_co2_per_kwh = grid.co2_per_kwh
             cost_co2 = [grid.cost_per_unit_co2]
 
             grid_max_import, grid_max_export = grid.max_import, grid.max_export
@@ -924,6 +950,19 @@ class ModelPredictiveControl:
         )
 
     def mpc_single_step(self, sample, previous_output, current_step):
+        """
+        :meta private:
+
+        Parameters
+        ----------
+        sample
+        previous_output
+        current_step
+
+        Returns
+        -------
+
+        """
 
         if not isinstance(previous_output, ControlOutput):
             raise TypeError('previous_output must be ControlOutput, unless first_step is True')
@@ -967,7 +1006,7 @@ class ModelPredictiveControl:
             genset_co2 = 0
 
         # Solve one step of MPC
-        control_dicts = self.set_and_solve(sample.loc[current_step:current_step + horizon - 1, 'load'].values,
+        control_dicts = self._set_and_solve(sample.loc[current_step:current_step + horizon - 1, 'load'].values,
                                           sample.loc[current_step:current_step + horizon - 1, 'pv'].values, temp_grid, price_import,
                                           price_export, e_max, e_min, p_max_charge, p_max_discharge, p_max_import,
                                           p_max_export, soc_0, p_genset_max, cost_co2, grid_co2, genset_co2, iteration=current_step, return_steps=self.microgrid.horizon)
